from dataclasses import dataclass

import psutil


@dataclass
class CPUInfo:
    core: int
    """CPU 物理核心数"""
    usage: float
    """CPU 占用"""
    freq: float
    """CPU 的时钟速度（单位：GHz）"""

    @classmethod
    def get_cpu_info(cls):
        cpu_core = psutil.cpu_count(logical=False)
        cpu_usage = psutil.cpu_percent(interval=1)
        cpu_freq = psutil.cpu_freq().current / 1000

        return CPUInfo(core=cpu_core, usage=cpu_usage, freq=cpu_freq)


@dataclass
class RAMInfo:
    """RAM 信息（单位：GB）"""

    total: float
    """RAM 总量"""
    usage: float
    """当前 RAM 占用"""

    @classmethod
    def get_ram_info(cls):
        ram_total = round(psutil.virtual_memory().total / (1024**3), 2)
        ram_usage = round(psutil.virtual_memory().used / (1024**3), 2)

        return RAMInfo(total=ram_total, usage=ram_usage)


@dataclass
class SwapMemory:
    """Swap 信息（单位：GB）"""

    total: float
    """Swap 总量"""
    usage: float
    """当前 Swap 占用"""

    @classmethod
    def get_swap_info(cls):
        swap_total = round(psutil.swap_memory().total / (1024**3), 2)
        swap_usage = round(psutil.swap_memory().used / (1024**3), 2)

        return SwapMemory(total=swap_total, usage=swap_usage)


@dataclass
class DiskInfo:
    """硬盘信息"""

    total: float
    """硬盘总量"""
    usage: float
    """当前硬盘占用"""

    @classmethod
    def get_disk_info(cls):
        disk_total = round(psutil.disk_usage("/").total / (1024**3), 2)
        disk_usage = round(psutil.disk_usage("/").used / (1024**3), 2)

        return DiskInfo(total=disk_total, usage=disk_usage)


def get_status_info() -> tuple[CPUInfo, RAMInfo, SwapMemory, DiskInfo]:
    """获取 `CPU` `RAM` `SWAP` `DISK` 信息"""
    cpu_info = CPUInfo.get_cpu_info()
    ram_info = RAMInfo.get_ram_info()
    swap_info = SwapMemory.get_swap_info()
    disk_info = DiskInfo.get_disk_info()

    return cpu_info, ram_info, swap_info, disk_info



import platform
from io import BytesIO

import nonebot
from nonebot import get_plugin_config
from nonebot.config import Config as BotConfig
from nonebot import __version__ as __nb_version__
from PIL import Image, ImageDraw, ImageFont

from . import __version__ as __status_version__
from .model import get_status_info
from .utils import truncate_string
from .path import (
    bg_img_path,
    adlam_font_path,
    baotu_font_path,
    marker_img_path,
    spicy_font_path,
)
from .color import (
    cpu_color,
    ram_color,
    disk_color,
    swap_color,
    details_color,
    nickname_color,
    transparent_color,
)

bot_config = get_plugin_config(BotConfig)

system = platform.uname()
nickname = list(bot_config.nickname)[0] if bot_config.nickname else "unknown"

adlam_fnt = ImageFont.truetype(str(adlam_font_path), 36)
spicy_fnt = ImageFont.truetype(str(spicy_font_path), 38)
baotu_fnt = ImageFont.truetype(str(baotu_font_path), 64)


def draw() -> bytes:
    """绘图"""

    loaded_plugins = nonebot.get_loaded_plugins()

    with Image.open(bg_img_path).convert("RGBA") as base:
        img = Image.new("RGBA", base.size, (0, 0, 0, 0))
        marker = Image.open(marker_img_path)

        cpu, ram, swap, disk = get_status_info()

        cpu_info = f"{cpu.usage}% - {cpu.freq}Ghz [{cpu.core}core]"
        ram_info = f"{ram.usage} / {ram.total} GB"
        swap_info = f"{swap.usage} / {swap.total} GB"
        disk_info = f"{disk.usage} / {disk.total} GB"

        content = ImageDraw.Draw(img)
        content.text((103, 581), nickname, font=baotu_fnt, fill=nickname_color)
        content.text((251, 772), cpu_info, font=spicy_fnt, fill=cpu_color)
        content.text((251, 927), ram_info, font=spicy_fnt, fill=ram_color)
        content.text((251, 1081), swap_info, font=spicy_fnt, fill=swap_color)
        content.text((251, 1235), disk_info, font=spicy_fnt, fill=disk_color)

        content.arc(
            (103, 724, 217, 838),
            start=-90,
            end=(cpu.usage * 3.6),
            width=115,
            fill=cpu_color,
        )
        content.arc(
            (103, 878, 217, 992),
            start=-90,
            end=(ram.usage * 3.6),
            width=115,
            fill=ram_color,
        )
        content.arc(
            (103, 1032, 217, 1146),
            start=-90,
            end=(swap.usage * 3.6),
            width=115,
            fill=swap_color,
        )
        content.arc(
            (103, 1186, 217, 1300),
            start=-90,
            end=(disk.usage * 3.6),
            width=115,
            fill=disk_color,
        )

        content.ellipse((108, 729, 212, 833), width=105, fill=transparent_color)
        content.ellipse((108, 883, 212, 987), width=105, fill=transparent_color)
        content.ellipse((108, 1037, 212, 1141), width=105, fill=transparent_color)
        content.ellipse((108, 1192, 212, 1295), width=105, fill=transparent_color)

        content.text(
            (352, 1378),
            f"{truncate_string(platform.processor())}",
            font=adlam_fnt,
            fill=details_color,
        )
        content.text(
            (352, 1431),
            f"{system.system} {system.release}",
            font=adlam_fnt,
            fill=details_color,
        )
        content.text(
            (352, 1484),
            f"NoneBot {__nb_version__} x Plugin {__status_version__}",
            font=adlam_fnt,
            fill=details_color,
        )
        content.text(
            (352, 1537),
            f"{len(loaded_plugins)} loaded",
            font=adlam_fnt,
            fill=details_color,
        )

        nickname_length = baotu_fnt.getlength(nickname)
        img.paste(marker, (103 + int(nickname_length) + 44, 595), marker)

        out = Image.alpha_composite(base, img)

        byte_io = BytesIO()
        out.save(byte_io, format="png")
        img_bytes = byte_io.getvalue()

        return img_bytes



from pathlib import Path

resources_dir: Path = Path(__file__).parent / "resources"

marker_img_path: Path = resources_dir / "images" / "marker.png"
bg_img_path: Path = resources_dir / "images" / "background.png"

baotu_font_path: Path = resources_dir / "fonts" / "baotu.ttf"
spicy_font_path: Path = resources_dir / "fonts" / "SpicyRice-Regular.ttf"
adlam_font_path: Path = resources_dir / "fonts" / "ADLaMDisplay-Regular.ttf"



from pydantic import Field, BaseModel

class ScopedConfig(BaseModel):

    only_superuser: bool = Field(default=False)
    """是否只能由 SUPERUSER 触发指令"""
    to_me: bool = Field(default=False)
    """触发指令是否需要 @Bot"""

class Config(BaseModel):
    status: ScopedConfig = Field(default_factory=ScopedConfig)
    """Kawaii Status Config"""



"""运行状态"""

from nonebot.rule import Rule
from nonebot.permission import SUPERUSER
from nonebot import require, get_plugin_config
from nonebot.plugin import PluginMetadata, inherit_supported_adapters

require("nonebot_plugin_alconna")
from nonebot_plugin_alconna import Command
from nonebot_plugin_alconna.uniseg import UniMessage

from .config import Config, ScopedConfig

__version__ = "0.1.2"
__plugin_meta__ = PluginMetadata(
    name="运行状态",
    description="NoneBot2 服务器状态查看插件",
    usage="/status",
    type="application",
    homepage="https://github.com/KomoriDev/nonebot-plugin-kawaii-status",
    config=Config,
    supported_adapters=inherit_supported_adapters("nonebot_plugin_alconna"),
    extra={
        "unique_name": "KaWaii Status",
        "example": "/status",
        "author": "Komorebi <mute231010@gmail.com>",
        "version": __version__,
    },
)

from .drawer import draw

config: ScopedConfig = get_plugin_config(Config).status


def to_me() -> Rule:
    if config.to_me:
        from nonebot.rule import to_me

        return to_me()

    async def _to_me() -> bool:
        return True

    return Rule(_to_me)


status = (
    Command("status", help_text="查看林汐运行状态")
    .usage("/status\n/状态")
    .action(lambda: UniMessage.image(raw=draw()))
    .build(
        rule=to_me(),
        aliases={"状态"},
        use_cmd_start=True,
        permission=SUPERUSER if config.only_superuser else None,
    )
)



def truncate_string(string: str, length: int = 32):
    """
    将字符串截断为给定长度。

    参数：
        string (str)：要截断的字符串。
        length (int)：截断字符串的最大长度。

    返回：
        str：截断的字符串。
    """
    if len(string) > length:
        return string[: length - 3] + "..."
    else:
        return string



from typing import Literal, TypeAlias

Color: TypeAlias = Literal[
    "cpu", "ram", "swap", "disk", "nickname", "details", "transparent"
]

cpu_color: tuple[int, int, int, int] = (84, 173, 255, 255)
ram_color: tuple[int, int, int, int] = (255, 179, 204, 255)
swap_color: tuple[int, int, int, int] = (251, 170, 147, 255)
disk_color: tuple[int, int, int, int] = (184, 170, 159, 255)
transparent_color: tuple[int, int, int, int] = (0, 0, 0, 0)
details_color: tuple[int, int, int, int] = (184, 170, 159, 255)
nickname_color: tuple[int, int, int, int] = (84, 173, 255, 255)


def get_color(color: Color) -> tuple[int, int, int, int]:
    return globals()[f"{color}_color"]
